﻿/*
* QuickFox Notes by Amin E (InBasic)
* eMail: inb.cor@gmail.com
*
* Source Code License: Mozilla Public License, version 1.1
*/

Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");

function _(aID) {
  return document.getElementById(aID);
}

var QuickFoxNotes = {
	currentNumberOfPanels: 0,	//Number of available panels
	mouseDownTab: null,			//This is the parent element for rename-panel
	intervalID: null,			//Autosave interval id
	//Elements
	window: null,
	tabs: null,
	tabpanels: null,
	tabbox: null,
	notifyBox: null,
	stringsBundle: null,
	//Word suggestion
	wordList: null,
	suggestMode: 0,				//In suggestion mode 1,2,3..,9 are suggestion keyboards
	//Error mode
	errorMode: false,			//Whenever errorMode is one, QFN will not save notes
	//Copy History
	copyHistory: [],
	//Commands
	trashCommand: "\
		var obj = QuickFoxNotes.addSingleTab([['value', value]]);\
		QuickFoxNotes.stylish.write(obj._tab, {label: label.match(/.*$/)[0], style: ''});\
		this.parentNode.removeChild(this);\
		var menupopup = _('editor.toolbar7.menupopup');\
		if (menupopup.childNodes.length == 2)\
			menupopup.parentNode.disabled = true;\
	",
	archiveCommand: "\
		var obj = QuickFoxNotes.addSingleTab([['value', getAttribute('value')], ['saveMe', true]], [['saveMe', true]]);\
		obj._textbox.editor.transactionManager.clear();\
		obj._textbox.setSelectionRange (0, 0);\
		obj._tab.bookmarkId = this.bookmarkId;\
		obj._tab.modifyPosition = true;\
		if (getAttribute('readonly') == 'true') {\
			obj._textbox.setAttribute('readonly', true);\
			obj._tab.setAttribute('readonly', true);\
		}\
		QuickFoxNotes.stylish.write(obj._tab, {label:QuickFoxNotes.group.read(this), style:QuickFoxNotes.stylish.readStyle(this)});\
		this.parentNode.removeChild(this);\
		var menupopup = _('editor.toolbar6.menupopup');\
		if (!menupopup.childNodes.length)\
			menupopup.parentNode.disabled = true;\
	",
	
	annotationService: Components.classes["@mozilla.org/browser/annotation-service;1"].
			getService(Components.interfaces.nsIAnnotationService),
	bmService: Components.classes["@mozilla.org/browser/nav-bookmarks-service;1"].
			getService(Components.interfaces.nsINavBookmarksService),
	prefs: Components.classes["@mozilla.org/preferences-service;1"]
                    .getService(Components.interfaces.nsIPrefService).getBranch("extensions.InBasic.QuickFoxNotes."),
	prompts: Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
								.getService(Components.interfaces.nsIPromptService),
	jsDump: Components.classes['@mozilla.org/consoleservice;1'].getService(Components.interfaces.nsIConsoleService),
	nativeJSON: Components.classes["@mozilla.org/dom/json;1"].createInstance(Components.interfaces.nsIJSON),
	
	/**Group*/
	group: {
		read: function (element) {
			if ("iGroup" in element && element.iGroup)
				return element.iGroup + "/" + element.iLabel;
			else
				return element.getAttribute ("label");
		},
		write: function (element, str) {
			const label = str.match(/[^\/]*$/)[0];
			var group = "";
			if (str.length != label.length)	//Group is not empty; remove "/"
				group = str.substr(0, str.length - label.length - 1);
			
			element.iGroup = group;
			element.iLabel = label;
			//Visual part
			if (element.localName == "menuitem" && group) {	//Archive element with non empty group
				element.setAttribute ("label", 
					QuickFoxNotes.prefs.getBoolPref("showGroupInArchive") ? "[" + group + "] " + label : label);
			}
			else if (element.localName == "tab" && group) {	//Tab element with non empty group
				element.setAttribute ("label", 
					QuickFoxNotes.prefs.getBoolPref("showGroupInTab") ? "[" + group + "] " + label : label);
			}
			else
				element.setAttribute ("label", label);
		},
		combine: function (strGroup, strLabel) {
			return strGroup + "/" + strLabel;
		},
		readLabel: function (element) {
			if ("iLabel" in element)
				return element.iLabel;
			else
				return element.getAttribute ("label");
		},
		readGroup: function (element) {
			if ("iGroup" in element)
				return element.iGroup;
			else
				return "";
		},
		readGroups: function () {
			var gorups = new Array();
			var group;
			
			var tab = QuickFoxNotes.tabs.getElementsByTagName("tab");
			for (i = 1; i < QuickFoxNotes.currentNumberOfPanels; i++) {		//Fisrt note is dummy
				group = QuickFoxNotes.group.readGroup(tab[i]);
				if (group && gorups.indexOf(group) == -1)
					gorups.push(group);
			}
			menupopup = _("editor.toolbar6.menupopup");
			for (var i = 0 ; i < menupopup.childNodes.length ; i++) {
				group = QuickFoxNotes.group.readGroup(menupopup.childNodes[i]);
				if (group && gorups.indexOf(group) == -1)
					gorups.push(group);
			}
			return gorups;
		},		
		/**Refresh label of Tabs & archives*/
		refresh: function () {
			//Refresh Tabs
			for (var tab = QuickFoxNotes.tabs.firstChild; tab; tab = tab.nextSibling)
				this.write (tab, this.read(tab));
			var menupopup = _("editor.toolbar6.menupopup");
			for (var menuitem = menupopup.firstChild; menuitem; menuitem = menuitem.nextSibling)
				this.write (menuitem, this.read(menuitem));
		}
	},
	/**Stylish and labeling tool*/
	stylish: {
		_compiler: function (str) {
			var temp = str.match(/\[style=(.*)\]/);
			
			if (temp) {
				return {
					label: str.replace(temp[0], ""),
					style: temp[1]
				}
			}
			else {
				return {
					label: str,
					style: ""
				}
			}
		},
		write: function (element, str) {
			var tempLabel =  str
			if (typeof(str) != "object")
				var tempLabel = this._compiler(str);
			
			if (element.localName == "menuitem") {	//Archive element
				QuickFoxNotes.group.write(element, tempLabel.label);	//Write group and label of element
				element.styleCode = tempLabel.style;
			}
			else if (element.localName == "tab") {	//Tab element
				QuickFoxNotes.group.write(element, tempLabel.label);	//Write group and label of element
				element.styleCode = tempLabel.style;
				element.setAttribute("style", tempLabel.style);
				element.style.minWidth = QuickFoxNotes.prefs.getCharPref("TabWidth") + "px";
			}
			else {
				alert("QuickFox Notes> stylish unsupported element");
			}
		},
		toString: function (element) {
			if ("styleCode" in element && element.styleCode) 
				return QuickFoxNotes.group.read(element) + "[style=" + element.styleCode + "]";
			else
				return QuickFoxNotes.group.read(element);
		},
		readStyle: function (element) {
			if ("styleCode" in element && element.styleCode)
				return element.styleCode;
			else
				return null;
		},
		readColor: function (element) {
			if ("styleCode" in element && element.styleCode) {
				var color = element.styleCode.match (/color[^:]*\:([^;]*)/);
				color = color ? color[1] : null;
				if (color)
					return color.replace(/ /g, '');
				else
					return "#000000";
			}
			else
				return "#000000";
		},
		readItalic: function (element) {
			if ("styleCode" in element && element.styleCode) {
				var fontStyle = element.styleCode.match (/font-style[^:]*\:([^;]*)/);
				fontStyle = fontStyle ? fontStyle[1] : "";
				return (/italic/.test(fontStyle));
			}
			else
				return false;
		},
		readBold: function (element) {
			if ("styleCode" in element && element.styleCode) {
				var fontWeight = element.styleCode.match (/font-weight[^:]*\:([^;]*)/);
				fontWeight = fontWeight ? fontWeight[1] : "";
				return (/bold/.test(fontWeight));
			}
			else
				return false;
		}
	},
	/**Create new notepad*/
	createNewPanel: function (name, dummy) {
		//add new tab
		var tab = document.createElement("tab");
		this.tabs.appendChild(tab);
		tab.setAttribute("context", "editor-context");
		tab.setAttribute("ondblclick", "if (event.button == 0) {QuickFoxNotes.onTabDBClick(); event.stopPropagation();}");	//Prevent event propagation to refuse create new tab by tabs.ondblclick
		tab.setAttribute("onmousedown", "QuickFoxNotes.mouseDownTab = this;");
		tab.setAttribute("onmouseup", "if (event.button == 1) QuickFoxNotes.closeSingleTab(true);"); //event.button == 1 >> middle click 
		tab.setAttribute("label", 
						 name ? name : "Note #" + this.tabpanels.getElementsByTagName("tabpanel").length);
		//Drag & drop only works for FF 3.6.0 or higher version
		var appInfo = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULAppInfo);//Detecting FF version
		if (!dummy && appInfo.version>"3.6.0") {
			tab.setAttribute("ondraggesture", "nsDragAndDrop.startDrag(event, QuickFoxNotes.dragNdrop.dragObserver)");
			tab.setAttribute("ondragexit", "nsDragAndDrop.dragExit(event, QuickFoxNotes.dragNdrop.dragObserver)");
			tab.setAttribute("ondragover", "nsDragAndDrop.dragOver(event, QuickFoxNotes.dragNdrop.dropObserver)");
			tab.setAttribute("ondragdrop", "nsDragAndDrop.drop(event, QuickFoxNotes.dragNdrop.dropObserver)");
		}
		//add new panel
		var tabpanel = document.createElement("tabpanel");
		tabpanel.setAttribute("flex", "1");
		var textbox = document.createElement("textbox");
		textbox.setAttribute("flex", "1");
		textbox.setAttribute("multiline", "true");
		textbox.setAttribute("spellcheck", true);	//Force to load dictionaries
		textbox.setAttribute("wrap", this.prefs.getBoolPref("wordWrap") ? "on" : "off");	//Force to load dictionaries
		textbox.setAttribute("class", "notepad");
		textbox.setAttribute("context", "editor_menupopup");
		textbox.style.fontSize = this.prefs.getCharPref("FontSize") + "px";
		textbox.style.fontFamily = this.prefs.getComplexValue("FontFamily", Components.interfaces.nsISupportsString).data;
		textbox.style.color = this.prefs.getCharPref("TextColor");
		textbox.style.backgroundColor = this.prefs.getCharPref("TabColor");
		tabpanel.style.backgroundColor = this.prefs.getCharPref("TabColor");
		textbox.addEventListener("mousedown", function (event) {QuickFoxNotes.mouseDown(event);}, false); 	//Paste on middle click
		textbox.addEventListener("select",    function () {QuickFoxNotes.copySelected(textbox)}, false); 	//Auto copy
		textbox.addEventListener("input", 	  function () {textbox.saveMe = true;}, false); 
		textbox.addEventListener("dragover",  function () {textbox.saveMe = true;}, false); 
		textbox.addEventListener("dblclick", function () {QuickFoxNotes.dbclick(textbox);}, false);
		//Add dictionary  
		if ((document instanceof Components.interfaces.nsIDOMXULDocument) &&
			(textbox instanceof Components.interfaces.nsIDOMXULTextBoxElement)) {
			
			var loader = Components.classes['@mozilla.org/moz/jssubscript-loader;1'].getService(Components.interfaces.mozIJSSubScriptLoader);
			loader.loadSubScript('chrome://global/content/inlineSpellCheckUI.js', this);
			if ('InlineSpellCheckerUI' in this)
				this.InlineSpellCheckerUI.init(textbox .editor);

			textbox.spellCheckerUI = this.InlineSpellCheckerUI;
		}
		tabpanel.appendChild(textbox);
		
		setTimeout( function() {
			try {
				textbox.controllers.insertControllerAt(0, QuickFoxNotes.contextmenu.controller);
				//Enable/disable spell checker
				textbox.editor.setSpellcheckUserOverride(QuickFoxNotes.prefs.getBoolPref("SpellCheck"));
			} catch (e){QuickFoxNotes.jsDump.logStringMessage("QuickFox Notes: [e][textbox.controllers][e:" + e.message + "]");}
		}, 100);

		this.tabpanels.appendChild(tabpanel);
		return {
			_textbox: textbox,
			_tab: tab
		};
	},
	/**Attribute manager*/
	attributeMG: {
		extractTitle: function (str) {
			var temp;
			try {
				temp = str.replace(/\[sb\]/gi, "")		//Remove sandbox option
						  .replace(/\[r\]/gi, "")		//Remove readonly option
						  .replace(/\[ss=\d*\]/gi, "")	//Remove selection start option
						  .replace(/\[se=\d*\]/gi, "")	//Remove selection end option
						  .replace(/\[st=\d*\]/gi, "")	//Remove scroll top option
						  .replace(/^[\ ]*/, "");		//L-Trim spaces
			} catch (e) {
				temp = str;
			}
			return temp;
		},
		check: function (str) {
			var tempScrollTop, tempSelectionStart, tempSelectionEnd;
			try {
				tempScrollTop = /\[st=(\d*)\]/.exec(str)[1];
			} catch(e) {
				tempScrollTop = 0;
			}
			try {
				tempSelectionStart = /\[ss=(\d*)\]/.exec(str)[1];
			} catch(e) {
				tempSelectionStart = 0;
			}
			try {
				tempSelectionEnd = /\[se=(\d*)\]/.exec(str)[1];
			} catch(e) {
				tempSelectionEnd = 0;
			}
			
			return {
				isArchive: /\[sb\]/i.test(str),
				isReadonly: /\[r\]/i.test(str),
				selectionStart: tempSelectionStart,
				selectionEnd: tempSelectionEnd,
				scrollTop: tempScrollTop
			}
		},
		toString: function (obj) {
			var temp = "";
			
			if ("isArchive" in obj && obj.isArchive)
				temp += (temp ? " " : "") + "[sb]";
			if ("isReadonly" in obj && (obj.isReadonly == "true" || obj.isReadonly == true))
				temp += (temp ? " " : "") + "[r]";
			if ("scrollTop" in obj && obj.scrollTop)
				temp += (temp ? " " : "") + "[st=" + obj.scrollTop + "]";
			if ("selectionStart" in obj && obj.selectionStart)
				temp += (temp ? " " : "") + "[ss=" + obj.selectionStart + "]";
			if ("selectionEnd" in obj && obj.selectionEnd)
				temp += (temp ? " " : "") + "[se=" + obj.selectionEnd + "]";
			
			if (temp)
				return temp + " ";
			else
				return "";
		}
	},
	/**Construct all available panels after load*/
	constructPanels: function () {
		var temp = 1;	//Fist note is dummy
		var tempLabel = "";
		//Add one dummy note
		var dummyTab = this.createNewPanel ("Dummy Note", true);
		dummyTab._tab.collapsed = true;	//Hide dummy note
		dummyTab._tab.disabled = true;	//Make it disabled so it is not reflected by Ctrl+Tab, scroll, ...
		//Add notes
		var sandBox = new Array();
		
		var list = this.io.getList();
		for (var i = 0; i < list.length; i++) {
			var title = this.io.read(this.io.ioTitle, list[i]);
			var description = this.io.read(this.io.ioDescription, list[i]);
			
			var att = this.attributeMG.check(title);
			//This item is for sandbox
			//Add to sandBox Part 1/2
			if (att.isArchive) {
				sandBox.push ({
					title: this.attributeMG.extractTitle(title),
					value: description,
					isReadonly: att.isReadonly,
					bookmarkId: list[i]
				});
			}
			else {
				var obj = this.createNewPanel (null);
				this.stylish.write (obj._tab, this.attributeMG.extractTitle(title));
				obj._tab.bookmarkId = list[i];	//Store bookmark id
				obj._tab._scrollTop = att.scrollTop;
				obj._tab._selectionStart = att.selectionStart;
				obj._tab._selectionEnd = att.selectionEnd;
				obj._textbox.setAttribute("value", description);
				//Set readonly attribute
				if (att.isReadonly) {
					obj._textbox.setAttribute("readonly", true);
					obj._tab.setAttribute("readonly", true);
				}
				//Some reports on obj._textbox.setSelectionRange is not a function
				setTimeout(function(t, index, att) {
					//Add awaiting note to the end of current note					
					var tempText = QuickFoxNotes.prefs.getComplexValue("pasteMeAfterInit", Components.interfaces.nsISupportsString).data;
					if (tempText && index == QuickFoxNotes.prefs.getIntPref("selectedTab") && (t.getAttribute('readonly') != 'true')) {
						var selStart = t.value.length;
						t.value += tempText;
						t.setSelectionRange(selStart, selStart + tempText.length)
						t.editor.selectionController.scrollSelectionIntoView(1 ,1 , true);
						t.saveMe = true;
						
						QuickFoxNotes.prefs.setCharPref("pasteMeAfterInit", "");
					}
					else {
						if (tempText && index == QuickFoxNotes.prefs.getIntPref("selectedTab")) {
							alert(QuickFoxNotes.stringsBundle.getString('canNotPaste'));
							QuickFoxNotes.prefs.setCharPref("pasteMeAfterInit", "");
						}
						t.setSelectionRange(att.selectionStart, att.selectionEnd);
						t.inputField.scrollTop = att.scrollTop;
					}
					t.editor.transactionManager.clear();	//Clear undo buffer
				}, 100, obj._textbox, temp, att);
				
				temp++;
			}
		}
		this.currentNumberOfPanels = temp;
		//Add to sandBox Part 2/2
		if (this.prefs.getBoolPref("sortArchive"))
			sandBox.sort (function(a, b) {	//If a or b does not belong to any group put them all together in '000' group
				var _a = (/\//.test(a.title) ? a.title : String.fromCharCode(0, 0, 0) + a.title);
				var _b = (/\//.test(b.title) ? b.title : String.fromCharCode(0, 0, 0) + b.title);
				return ((_a.toLowerCase() > _b.toLowerCase()) ? +1 : -1);
			});	//Sort array!
		var menupopup = _("editor.toolbar6.menupopup");
		for (var i = 0; i < sandBox.length; i++) {
				var menuitem = document.createElement("menuitem");
				this.stylish.write (menuitem, sandBox[i].title);
				menuitem.setAttribute("value", sandBox[i].value);
				menuitem.setAttribute("readonly", sandBox[i].isReadonly);
				menuitem.bookmarkId = sandBox[i].bookmarkId;	//Store bookmark id
				menuitem.setAttribute("oncommand", this.archiveCommand);
				
				menupopup.appendChild(menuitem);
		}
		if (sandBox.length)
			menupopup.parentNode.disabled = false; //Make sure that menupopup is enabled
		//
		if (this.prefs.getIntPref("selectedTab") > (this.tabs.itemCount - 1) )
			this.tabbox.selectedIndex = 0;	//QFN configuration change by xmarks update!
		else
			this.tabbox.selectedIndex = this.prefs.getIntPref("selectedTab");
		
		if (temp == 1)	//First Tab is dummy
			this.addSingleTab();  //Add single tab if there isnt any available
	},
	/**Save preferences*/
	onBeforeUnload: function () {
		for (var i = 1; i < this.currentNumberOfPanels; i++) {		//First Tab is dummy
			var textbox = this.getEditor(i); 
			var tab = this.tabs.childNodes[i];
			if ( tab._scrollTop != textbox.inputField.scrollTop ||
				 tab._selectionStart != textbox.selectionStart ||
				 tab._selectionEnd != textbox.selectionEnd )
				tab.saveMe = true;
				
			tab._scrollTop = textbox.inputField.scrollTop;
			tab._selectionStart = textbox.selectionStart;
			tab._selectionEnd = textbox.selectionEnd;
		}
	},
	/**Store all data*/
	savePanels: function () {
		//Do not save anything in Safemode
		if (this.errorMode) 
			return;
		//Create Report	
		var report = "";
		//Save Clipboards
		var str = Components.classes["@mozilla.org/supports-string;1"]
			.createInstance(Components.interfaces.nsISupportsString);
		str.data = this.nativeJSON.encode(this.copyHistory);
		this.prefs.setComplexValue("clipboard", Components.interfaces.nsISupportsString, str); 
		//Save Trash
		var menupopup = _("editor.toolbar7.menupopup");
		var trashData = new Array();
		
		for (var trashItem = menupopup.firstChild; trashItem &&  trashItem.id != "editor.toolbar7.menupopup.trash"; trashItem = trashItem.nextSibling) {	//Do not save "clear trash" item
			
			trashData.push({
				label: trashItem.getAttribute("label"),
				value: trashItem.getAttribute("value")
			});
		}
		
		str.data = this.nativeJSON.encode(trashData);
		this.prefs.setComplexValue("trashData", Components.interfaces.nsISupportsString, str); 
		//Save Archive, Tabs
		var tab = this.tabs.getElementsByTagName("tab");
		//available ids
		var availableIDs = this.io.getList();

		//Save Tabs
		for (var i = 1; i < this.currentNumberOfPanels; i++) {		//First Tab is dummy
			var bookmarkId = tab[i].bookmarkId;
			var title = this.attributeMG.toString({isArchive: false, 
			   isReadonly: tab[i].getAttribute('readonly'),
			   scrollTop: tab[i]._scrollTop,
			   selectionStart: tab[i]._selectionStart,
			   selectionEnd: tab[i]._selectionEnd
			}) + this.stylish.toString (tab[i]);
			var description = this.getEditor(i).value;

			//Always save title before description; since title will create new id if bookmark does not exist
			if (tab[i].saveMe) {
				bookmarkId = this.io.write (this.io.ioTitle, bookmarkId, title) || bookmarkId;	//If bookmarkId == "" new bookmark will created and bookmarkId will be updated
				if (!bookmarkId)	//This happens when QFN is in local mode and file index is 0
					bookmarkId = 0;
				tab[i].bookmarkId = bookmarkId; //Associate the bookmark id to tab
				report += bookmarkId + "[T][wTitle]; ";
				tab[i].saveMe = false;
			}
			if (this.getEditor(i).saveMe) {
				this.io.write (this.io.ioDescription, bookmarkId, description);
				report += bookmarkId + "[T][wData]; ";
				this.getEditor(i).saveMe = false;
			}
			//This fires if item come from archive or it's location changed by drag & drop
			if (("modifyPosition" in tab[i]) && (tab[i].modifyPosition == true)) {
				var newLocation = 0;
				if (i != 1)	//First note is dummy!
					newLocation = this.io.getItemIndex(tab[i - 1].bookmarkId) + 1;
				this.io.moveItem(bookmarkId, newLocation);	//Change position of bookmark in bookmark folder
				
				tab[i].modifyPosition = false;
				
				report += bookmarkId + "[T][modifyLocation]; ";
			}
			//Remove id; remain ids will be deleted
			var isBookmarkAvailable = availableIDs.indexOf(bookmarkId);
			if (isBookmarkAvailable != -1)
				availableIDs.splice(isBookmarkAvailable, 1);	//Remove this bookmark
		}
		//Save Sandbox notes
		menupopup = _("editor.toolbar6.menupopup");
		for (var j = i ; j < (i + menupopup.childNodes.length) ; j++) {
			var bookmarkId = menupopup.childNodes[j - i].bookmarkId;
			var title = this.attributeMG.toString({isArchive: true, isReadonly: menupopup.childNodes[j - i].getAttribute("readonly")}) + 
						this.stylish.toString (menupopup.childNodes[j - i]);				
						
			var description = menupopup.childNodes[j - i].getAttribute("value");
			
			if (menupopup.childNodes[j - i].saveMe) {
				//Always save title before description; since title will create new id if bookmark does not exist
				bookmarkId = this.io.write (this.io.ioTitle, bookmarkId, title) || bookmarkId;
				menupopup.childNodes[j - i].bookmarkId = bookmarkId;	//Associate the bookmark id to menupopup
				this.io.write (this.io.ioDescription, bookmarkId, description);

				report += bookmarkId + "[S][wData + wTitle]; ";
				menupopup.childNodes[j - i].saveMe = false;
			}
			//Remove id; remain ids will be deleted
			var isBookmarkAvailable = availableIDs.indexOf(bookmarkId);
			if (isBookmarkAvailable != -1)
				availableIDs.splice(isBookmarkAvailable, 1);	//Remove this bookmark
		}
		
		//Remove Extra Bookmarks information!
		if (availableIDs.length)
			report += availableIDs.toString() + "[R]; ";
		this.io.remove(availableIDs);

		if (report)
			this.jsDump.logStringMessage("QuickFox Notes - save Report [" + this.getTime() + "]: " + report);
	},
	/**Close a notepad*/
	closeSingleTab: function (byMiddleMouse, forced) {
		//In the case where an inactive tab is removed (it can be possible by middle click on an inactive tab), renderTabs() is not called!
		//Find Tab
		var callRenderTabs = true;	//need to focus on new tab
		if (byMiddleMouse) {
			var index = this.tabs.getIndexOfItem (this.mouseDownTab);
			callRenderTabs = (this.mouseDownTab == this.tabs.selectedItem);	//Is to be removed tab same as selected tab or not
		}
		else 
			var index = this.tabs.selectedIndex;
 		//Confirm Close
		if (this.prefs.getBoolPref("showConfirmClose") && !forced) {
			var check = {value: false};
			var result = this.prompts.confirmCheck(null, this.stringsBundle.getString('confirmTitle'), 
											  this.stringsBundle.getString('confirmBody'),
											  this.stringsBundle.getString('confirmCheckMSG'), check);
			
			this.prefs.setBoolPref("showConfirmClose", !check.value);
			_("editor-tabbox-toolbar-menuitem5").setAttribute("checked", !check.value);
			if (!result)
				return;
		}
		//Do not close protected tabs
		if(this.tabs.getItemAtIndex(index).getAttribute('readonly')) {
			alert(this.stringsBundle.getString('doNotCloseProtectedTab'));
			return;
		}
		if (index >= 0) {
			if (!forced) {	//Do not Save data when it is going to the Archive 
				//Saving tab data before closing it
				function pad(n){return n < 10 ? '0' + n : n}
				var curTime = new Date();
				var menupopup = _("editor.toolbar7.menupopup");
				var menuitem = document.createElement("menuitem");
				menuitem.setAttribute("label", 
									  pad(curTime.getHours()) + ":" +  pad(curTime.getMinutes()) + ":" + pad(curTime.getSeconds()) + " - " + 
									  pad(curTime.getDate()) + "/" +  pad(curTime.getMonth() + 1) + "/" + curTime.getFullYear() + " | " +
									  String.fromCharCode (10) +	//Indicator
									  this.group.read(this.tabs.getItemAtIndex(index)));
				menuitem.setAttribute("value", this.getEditor(index).value);
				menuitem.setAttribute("oncommand", this.trashCommand);
				menupopup.insertBefore(menuitem, _("editor.toolbar7.menupopup.trash"));

				menupopup.parentNode.disabled = false; //Make sure that menupopup is enabled
			}
			//Closing tab
			this.tabpanels.removeChild(this.tabpanels.getElementsByTagName("tabpanel")[index]);
			this.tabs.removeItemAt(index);
			this.currentNumberOfPanels -= 1;
		}
		if (this.currentNumberOfPanels == 1) 	//First Tab is dummy
			this.addSingleTab();  //Add single tab if there isnt any available
		if (callRenderTabs)		// if changing focus is required
			this.tabbox.selectedIndex = (index > 0)? index - 1 : 0;	//Select new tab 
		else {
			this.tabbox.selectedIndex = this.tabs.selectedIndex;
			this.renderTabs();	//This function is not called only in the case where other than selected tab is removed
		}
		
		return this;
	},
	/**Create new notepad*/
	addSingleTab: function (textboxAttributes, tabAttributes) {
		var temp = this.createNewPanel(null);
		//calculate new Title
		var body = this.stringsBundle.getString('UnKnownTitle');
		var tab = this.tabs.getElementsByTagName("tab");
		//Find for a free title
		var index = 0;
		var used = true;
		while (used) {
			used = false;
			index += 1;
			for (var i = 1; i < this.currentNumberOfPanels; i++) {		//First Tab is dummy
				if (this.group.readLabel(tab[i]) == body + " " + index) {
					used = true;
					continue;
				}
			}
		}
		
		this.stylish.write(temp._tab, {label: body + " " + index, style: ""});
		
		this.setAttributes(temp._tab, [
			['bookmarkId', null], //Store bookmark id
			['saveMe', true]	  //This is new tab save it!					
		])
		temp._textbox.saveMe = true;	//Store bookmark id
		//Set external attributes
		if (textboxAttributes)
			this.setAttributes(temp._textbox, textboxAttributes)
		if (tabAttributes)
			this.setAttributes(temp._tab, tabAttributes)
		
		
		this.currentNumberOfPanels += 1;
		this.tabbox.selectedIndex = this.currentNumberOfPanels - 1;	//Select last panel

		return temp;
	},
	/**Update list of notes*/
	updateList: function () {
		var menupopup = _("editor.toolbar9.menupopup");
		//Remove old childs
		while (menupopup.firstChild)
			menupopup.removeChild(menupopup.firstChild);
		//Add new list
		var tab = this.tabs.getElementsByTagName("tab");
		for (var i = 1; i < this.currentNumberOfPanels; i++) {
			var menuitem = document.createElement("menuitem");
			menuitem.setAttribute("label", this.group.readLabel(tab[i]));
			menuitem.setAttribute("value", i);
			menuitem.setAttribute("oncommand", "QuickFoxNotes.tabbox.selectedIndex = this.value;");
			menupopup.appendChild(menuitem);
		}
	},
	/**Archive panel*/
	archiveII: {
		popup: function () {
			function modifyLabel (element, num) {
				element.setAttribute('value', element.getAttribute('value') + " (" + num + "):");
			}
		
			var arrowscrollbox = _("editor-archive-panel-arrowscrollbox");
			var menupopup = _("editor.toolbar6.menupopup");
			//Remove old childs
			while (arrowscrollbox.getElementsByTagName("label").length)
				arrowscrollbox.removeChild (arrowscrollbox.getElementsByTagName("label")[0]);
			//Add new items
			var oldGroup = "";
			var oldGroupElement = null;
			var i = 0;	//Total number of elements
			var j = 0;	//Number of elements in same group
			for (var menuitem = menupopup.firstChild; menuitem; menuitem = menuitem.nextSibling) {
				//Add group
				var group = QuickFoxNotes.group.readGroup(menuitem)
				if (group != oldGroup) {
					if (oldGroupElement)	//Modify text of previous group
						modifyLabel(oldGroupElement, j);
					j = 0;
					var label = document.createElement('label');
					oldGroup = group;
					oldGroupElement = label;
					label.setAttribute('class', "arItemGroup text-link");
					label.setAttribute('value', group);
					label.setAttribute('onclick', "QuickFoxNotes.removeAFArchive('" + group + "'); _('editor-archive-panel').hidePopup();");
					arrowscrollbox.appendChild(label);
				}
				//Add item
				var label = document.createElement('label');
				label.setAttribute('value', QuickFoxNotes.group.readLabel(menuitem));
				label.setAttribute('group', QuickFoxNotes.group.readGroup(menuitem));
				label.setAttribute('onclick', "_('editor.toolbar6.menupopup').childNodes[" + i + "].doCommand(); _('editor-archive-panel').hidePopup();");
				i++;
				j++;
				label.setAttribute('class', "arItemItem text-link");
				
				arrowscrollbox.appendChild(label);
			}
			if (oldGroupElement)	//Modify the last group's name
				modifyLabel(oldGroupElement, j);	
			
			this.filter(_("editor-archive-panel-textbox").value);
			_("editor-archive-panel-textbox").focus();
		},
		handleESC: function () {
			var searchbox = _('editor-archive-panel-textbox');
			try {
				if (document.activeElement.parentNode.parentNode == searchbox && searchbox.value) {
					searchbox.value = "";
					this.filter("");
					setTimeout(function(){searchbox.focus();}, 100)
					return false;
				}
			}
			catch (e) {}
		},
		filter: function(word) {
			function check (word, label, group) {
				var parts = word.match(/[^ ]+/g);
				
				if (!parts)
					return true;
				
				var isMatched = true;
				for (var i = 0; i < parts.length; i++) {
					var myRe = new RegExp (parts[i], "i"); 					
					isMatched = isMatched && (myRe.test(label) || myRe.test(group));
				}
				return isMatched;
			}
			var arrowscrollbox = _("editor-archive-panel-arrowscrollbox");
			var listitems = arrowscrollbox.getElementsByClassName("arItemItem");
			for (var i = 0; i < listitems.length; i++) {
				listitems[i].hidden = !check(word, listitems[i].getAttribute("value"), listitems[i].getAttribute("group"));			
			}
		}	
	},
	/**Remove all notes from Archive/Sandbox*/
	removeAFArchive: function (groupName) {
		var menupopup = _('editor.toolbar6.menupopup');
		
		var items = new Array();
		for (var i = menupopup.firstChild; i; i = i.nextSibling) {
			if (groupName) {
				if (this.group.readGroup(i) == groupName)
					items.push(i);
			}
			else
				items.push(i);
		}
		for (var j = 0; j < items.length; j++)
			setTimeout (function(item){item.doCommand();}, 200 * j, items[j]);
	},
	/**Clear Trash*/
	clearTrash: function () {
		if (window.confirm(this.stringsBundle.getString('clearAllTrashes'))) {
			var menupopup = _('editor.toolbar7.menupopup');
			while (menupopup.childNodes.length != 2) {
				var firstChild = menupopup.firstChild;
				menupopup.removeChild(menupopup.firstChild);
			}
			 
			menupopup.parentNode.disabled = true;
			this.prefs.setCharPref("trashData", "");
		}
	},
	insertTextAtCursorPoint: function (str, textbox, selStart, selEnd) {
		var command = "cmd_insertText";
		if (!str)	//when there is no string it means that delete current range of text
			var command = "cmd_delete";
		try {
			//changing range
			if (textbox) {
				textbox.selectionStart = selStart;
				textbox.selectionEnd = selEnd;
			}
			var controller = document.commandDispatcher.getControllerForCommand(command);
			if (controller && controller.isCommandEnabled(command)) {
			  controller = controller.QueryInterface(Components.interfaces.nsICommandController);
			  var params = Components.classes["@mozilla.org/embedcomp/command-params;1"];
			  params = params.createInstance(Components.interfaces.nsICommandParams);
			  params.setStringValue("state_data", str);
			  controller.doCommandWithParams(command, params);
			}
		}
		catch (e) {this.jsDump.logStringMessage("QuickFox Notes: [e][insertTextAtCursorPoint][e:" + e.message + "]");}

	},
	/**Add current note to the sandbox*/
	addToSandbox: function () {
		var menupopup = _("editor.toolbar6.menupopup");
		var menuitem = document.createElement("menuitem");
		
		this.stylish.write (menuitem, {label: this.group.read(this.tabs.selectedItem), style: this.stylish.readStyle(this.tabs.selectedItem)});
		
		menuitem.setAttribute("value", this.getEditor().value);
		menuitem.saveMe = true;
		menuitem.setAttribute("readonly", this.tabs.selectedItem.getAttribute("readonly"));
		menuitem.bookmarkId = this.tabs.selectedItem.bookmarkId;	//Store bookmark id
		
		menuitem.setAttribute("oncommand", this.archiveCommand);
		//Insert item at appropriate location
		var found = false;
		for (var i = menupopup.firstChild; i; i = i.nextSibling) {
			var _a = this.group.readGroup(i) ? this.group.read(i) : String.fromCharCode(0, 0, 0) + this.group.read(i);		//If not does not belong to any group add it to '000' group
			var _b = this.group.readGroup(this.tabs.selectedItem) ? this.group.read(this.tabs.selectedItem) : String.fromCharCode(0, 0, 0) + this.group.read(this.tabs.selectedItem);		//If not does not belong to any group add it to '000' group
			
			if (_a.toLowerCase() > _b.toLowerCase()) {
				found = true;
				break;
			}
		}
		if (found)
			menupopup.insertBefore (menuitem, i);
		else
			menupopup.appendChild(menuitem);
		//
		menupopup.parentNode.disabled = false; //Make sure that menupopup is enabled
		this.tabs.selectedItem.removeAttribute("readonly");
		this.closeSingleTab (false, true);
	},
	/**Handling mouse wheel for tabbox*/
	scrollOnMouse: function () {
		var tabContainer = _("editor-tabbox-tabcontainer");  
		tabContainer.addEventListener("DOMMouseScroll", function scroll(event){
			//event.detail is positive for a downward scroll, negative for an upward scroll
			QuickFoxNotes.tabs.advanceSelectedTab( (/\-/.test(event.detail) ? -1 : +1), !QuickFoxNotes.prefs.getBoolPref("ScrollStopOnEnd"));
		}, false);
	},
	renderTabs: function () {
		if (this.tabs.selectedIndex == 0)	//Fist Tab is dummy
			this.tabbox.selectedIndex = 1;
		document.title = "[" + (this.tabs.selectedIndex)+ "/" + (this.currentNumberOfPanels - 1) + "] " + 
						 (this.prefs.getIntPref('repositoryType') ? '[' + this.stringsBundle.getString('io14') + '] ' + this.prefs.getCharPref('sqlitePath') : '[' + this.stringsBundle.getString('io10') + '] ' + this.prefs.getCharPref('FolderName'));
		setTimeout (function () {
			QuickFoxNotes.getEditor().focus();	//Set focus to the textbox
		}, 100);	//On the case of dbclick on tabbar, focus need some time!
		//Refresh find
		this.find(true, true); 
		//Update selectedIndex
		this.prefs.setIntPref('selectedTab', this.tabs.selectedIndex);
		//Make sure that selected tab is shown
		var tabContainer = _('editor-tabbox-tabcontainer');
		tabContainer.ensureElementIsVisible(QuickFoxNotes.tabs.selectedItem);
		//Remove notifications
		this.notifyBox.removeAllNotifications( true );
		//Refresh Toolbar
		this.toolbar.controller.onpopupshowing(_("editor-tabbox-toolbar"));
		return this;
	},
	find: function (forward, newSearch){//True: look forward; False: Look backward
		var textbox = this.getEditor();
		var searchbox = _('editor-tabbox-toolbar-searchbox');

		var reg = new RegExp(
			function (text) {	//Escaping regular expression characters: http://simonwillison.net/2006/Jan/20/escape/
				if (!arguments.callee.sRE) {
					var specials = ["/", ".", "*", "+", "?", "|", "(", ")", "[", "]", "{", "}", "\\"]; 
					arguments.callee.sRE = new RegExp("(\\" + specials.join("|\\") + ")", "g"); 
				}
				return text.replace(arguments.callee.sRE, "\\$1"); 
			}(searchbox.value)	
			, "ig");
		//change style of 3 elements: search textbox, search buttons and icon's of tabs
		var changeStyle = function (searchButtons, searchTextbox, searchCount, previousCount) {
			//seach box
			searchbox.setAttribute("noresult", !searchTextbox);
			//search buttons
			_('editor-tabbox-toolbar-forward').setAttribute("disabled", !searchButtons);
			_('editor-tabbox-toolbar-backward').setAttribute("disabled", !searchButtons);
			//Change icon's of all tabs
			for (var i = 0; i < QuickFoxNotes.tabs.getElementsByTagName("tab").length; i++) {
				var tab = QuickFoxNotes.tabs.getElementsByTagName("tab")[i];
				if (searchbox.value) {
					var text = QuickFoxNotes.getEditor(i).value;
					reg.lastIndex = 0;
					if (reg.test(text))
						tab.setAttribute ("search", true);//search is a CSS attribute
					else
						tab.setAttribute ("search", false);
				}
				else 
					tab.removeAttribute ("search");//search is a CSS attribute
			}
 
			//Update search count
			if (searchCount > 0)
				_("editor-tabbox-toolbar-searchcount").value =
					previousCount + "/" + searchCount;
			else
				_("editor-tabbox-toolbar-searchcount").value = "0/0";
			
		};		
		
		if (searchbox.value) {	//search box is not empty
			//Find and highlight the phrase
			var position = textbox.selectionStart;
			if (textbox.selectionStart != textbox.selectionEnd && !newSearch)	//Prevent new search as same as old search
				position += (forward ? +1 : -1);
			if (position == textbox.value.length && forward) //Rotate for forward = true
				position = 0;
			if ((position == 0 || position == -1) && !forward) //Rotate for forward = false
				position = textbox.value.length;

			reg.lastIndex = 0;
			const regResult = textbox.value.match(reg);
			var numberOfMatches = regResult ? regResult.length : 0;
			
			var result = textbox.value.toLowerCase()[forward ? "indexOf" : "lastIndexOf"](searchbox.value.toLowerCase(), position);
			if (result == -1 && numberOfMatches > 0) {	//There is matched phrase in unsupported area
				const rePosition = forward ? 0 : textbox.value.length;
				var result = textbox.value.toLowerCase()[forward ? "indexOf" : "lastIndexOf"](searchbox.value.toLowerCase(), rePosition);
			}
			
			if (result != -1 && numberOfMatches > 0) {	// Successful search
				textbox.setSelectionRange(result, result + searchbox.value.length);
				//Scroll to selected area
				textbox.editor.selectionController.scrollSelectionIntoView(1 ,1 , true);
				//Make highlight visible
				textbox.editor.selectionController.setDisplaySelection(2);
				//Number of matches before
				const regParResult = textbox.value.substr(result).match(reg);
				var numberOfParMatches = regParResult ? regParResult.length : 0;
				//Updating visual
				changeStyle ((numberOfMatches >  1), true, numberOfMatches, numberOfMatches - numberOfParMatches + 1);
			}
			else if (result == -1 && numberOfMatches > 0)
				QuickFoxNotes.api.alert("find()", "This is a bug in search utility!");
			else {	//Not found
				textbox.setSelectionRange(position, position);
				//Updating visual
				changeStyle (false, false, 0, 0);
			}
		}
		else { //search box is empty
			//Updating visual
			changeStyle (false, true, 0, 0);
		}	
		return this;
 	},
	/**Backups*/
	backups: {	
		init: function () {
		//Backup database
			var oldDate = new Date(parseInt(QuickFoxNotes.prefs.getCharPref("lastBackup2"))); 
			var today = new Date();
			var dayDiff = Math.round((today.getTime() - oldDate.getTime())/(1000*60*60*24));	//1000*60*60*24 is one day
			if (dayDiff >= QuickFoxNotes.prefs.getIntPref("backupPeriod")) {
				QuickFoxNotes.prefs.setCharPref("lastBackup2", today.getTime());
				this.doBackup();
			}
		},
		foldersList: function(file) {
			var folderEnum = file.directoryEntries;
			var folders = [];
			while (folderEnum.hasMoreElements()) {
				var tmp = folderEnum.getNext().QueryInterface(Components.interfaces.nsILocalFile)
				if (tmp.isDirectory())
					folders.push(tmp);
			}
			return folders;
		},
		doBackup: function () {
			//Get backup folder
			var file = Components.classes["@mozilla.org/file/directory_service;1"].
					   getService(Components.interfaces.nsIProperties).
					   get("ProfD", Components.interfaces.nsIFile);
			file.append("qfn-backups");
			if( !file.exists() || !file.isDirectory() ) {   // if it doesn't exist, create
			   file.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0777);
			}
			//Delete old folders
			var numberOfBackups = QuickFoxNotes.prefs.getIntPref("numberOfBackups");
			var backupFolders = this.foldersList(file);
			for (var i = 0; i < backupFolders.length - numberOfBackups + 1; i++)
				backupFolders[i].remove(true);
			//Folder name
			var today = new Date();
			var month = today.getMonth() + 1;
			if (month < 10){month = "0" + month};
			var day = today.getDate();
			if (day < 10){day = "0" + day};
			var folderName = "Backup-" + today.getFullYear() + "-" + month + "-" + day;
			//Create a new folder and do backup
			file.append(folderName);
			if( !file.exists() || !file.isDirectory() ) {   // if it doesn't exist, create
			   file.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0777);
			}
			//Save all
			var osString = Components.classes["@mozilla.org/xre/app-info;1"]
				.getService(Components.interfaces.nsIXULRuntime).OS;
			var seperator = (/WIN/.test(osString) ? "\\" : "/");
			
			var tab = QuickFoxNotes.api.list.tab();
			for (var i = 0; i < tab.length; i++) {
				var data = QuickFoxNotes.api.get.noteContent(i + 1);
				var title =  QuickFoxNotes.api.get.noteTitle(tab[i]);
				QuickFoxNotes.api.saveToFile(file.path + seperator + title + ".txt", data);
			}
			var archive = QuickFoxNotes.api.list.archive();
			for (var i = 0; i < archive.length; i++) {
				var menuitem = archive[i];
				var data = QuickFoxNotes.api.get.archiveContent(menuitem);
				var title = QuickFoxNotes.api.get.archiveTitle(menuitem)
				QuickFoxNotes.api.saveToFile(file.path + seperator + "[archived] " + title + ".txt", data);
			}
		},
		reveal: function () {
			var file = Components.classes["@mozilla.org/file/directory_service;1"]. 
				getService(Components.interfaces.nsIProperties). 
				get("ProfD", Components.interfaces.nsILocalFile); 
			file.append("qfn-backups");
			file.reveal();
		}
	},
	/**Handling mouse functions for editors*/
	mouseDown: function (event) {
		//Paste on middle click
		if (event.button == 1 && this.prefs.getBoolPref("pasteOnMiddleClick")) {
			var clip = Components.classes["@mozilla.org/widget/clipboard;1"].getService(Components.interfaces.nsIClipboard);  
			if (!clip) return false;    
			var trans = Components.classes["@mozilla.org/widget/transferable;1"].createInstance(Components.interfaces.nsITransferable);  
			if (!trans) return false;  
			trans.addDataFlavor("text/unicode");  
			clip.getData(trans, clip.kGlobalClipboard);  
			   
			var str       = new Object();  
			var strLength = new Object();  
			  
			trans.getTransferData("text/unicode", str, strLength);  
			if (str) str = str.value.QueryInterface(Components.interfaces.nsISupportsString);  
			if (str) this.insertTextAtCursorPoint(str.data.substring(0, strLength.value / 2));
		}
	},
	/**Copy selected text from textbox*/
	copySelected: function (textbox) {
		if (this.prefs.getBoolPref("copySelected")) {
			setTimeout (function () {textbox.editor.copy();}, 200);
		}
	},
	/**Double click on Tab*/
	onTabDBClick: function () {
		this.addToSandbox ();
		var index = this.prefs.getIntPref("notifySandbox");
		if (index < 6) {
			this.notify (this.stringsBundle.getString('tabDBClick'));
			this.prefs.setIntPref("notifySandbox", index + 1);
		}
	},
	/**Disabling set on Top when QFN is minimized*/
	resize: function () {
		if(window.windowState == Components.interfaces.nsIDOMChromeWindow.STATE_MINIMIZED)
			QuickFoxNotes.alwaysOnTop(QuickFoxNotes.window, false);
		else
			QuickFoxNotes.alwaysOnTop(QuickFoxNotes.window, QuickFoxNotes.prefs.getBoolPref("alwaysOnTop"));
	},
	closeSidebarWindow: function () {
		var mainWindow = window.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
							   .getInterface(Components.interfaces.nsIWebNavigation)
							   .QueryInterface(Components.interfaces.nsIDocShellTreeItem)
							   .rootTreeItem
							   .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
							   .getInterface(Components.interfaces.nsIDOMWindow);
		mainWindow.document.getElementById("quickfox-notes-browser").setAttribute("src", "about:blank");
		mainWindow.document.getElementById("quickfox-notes-contentsplitter").setAttribute("collapsed", true);
		mainWindow.document.getElementById("quickfox-notes-contentbox").setAttribute("collapsed", true);
	},
	/**Closing QFN*/
	closeMe: function () {
		//Close Me
		if (_("editor.toolbar8.toolbarbutton").getAttribute("collapsed") == "false") //QFN is on application mode
			this.closeSidebarWindow();
		else
			window.close();
	},
	/**Realtime word suggest*/
	suggestWord: {
		detected: "",
		
		suggest: function () {
			if (this.detected) {
				var req = new XMLHttpRequest();
				req.open('GET', 'http://suggestqueries.google.com/complete/search?' + QuickFoxNotes.prefs.getCharPref("googleWords") + encodeURIComponent(this.detected), false);
				req.send(null);
				
				if(req.status == 200){
					var response = QuickFoxNotes.nativeJSON.decode(req.responseText);	//response is a structure with 2 arrays, second array is the word list
					
					if (response[1].length > 0) {
						var list = "";
						for (var i = 0; i < 9 && i < response[1].length; i++) {
							if (i == 0 && /^=\ /.test(response[1][i])) {//Check google calculator's result
								response[1][0] = this.detected + " " + ( (/=[\ \t]*$/.test(this.detected)) ? response[1][0].substr(1) : response[1][0]);
							}
							list += "[" + (i + 1) + "] " + response[1][i] + ((i != (Math.min(response[1].length, 9) - 1)) ? ", " : "");
						}
						QuickFoxNotes.notify (list, "google");
						QuickFoxNotes.wordList = response[1];
						QuickFoxNotes.suggestMode = 1;
						
						return;
					}
					else {
						QuickFoxNotes.notify (QuickFoxNotes.stringsBundle.getString('noResult'));
						QuickFoxNotes.suggestMode = 2;
						
						return;
					}
				}
				
			}
			//There is no suggestions
			QuickFoxNotes.notify (QuickFoxNotes.stringsBundle.getString('noResult'));
			QuickFoxNotes.suggestMode = 2;
		},
		detectWord: function (number){	//number of desire words
			var textbox = QuickFoxNotes.getEditor();	//Selected Textbox
			var selectionStart = textbox.selectionStart;

			var skip = 0;	//number of white spaces before 
			try {
				skip = textbox.value.substring(0, selectionStart).match(/[ \t]*$/)[0].length;
			} catch (e) {}
			
			var j = 0;
			for (var i = selectionStart - skip - 1; i >= Math.max(selectionStart - skip - 101, 0); i--) {	//only check 100 first chars! (due to speed issue)
				var ch = textbox.value.substr(i, 1);
				if ((ch == " " || ch == "\t") && number == 1) {
					j = i + 1;
					break;
				}
				if ((ch == " " || ch == "\t") && number > 1) 
					number--;
				if (ch == "\n"){
					j = i + 1;
					break;
				}
			}
			
			//var word = /[^ ]*$/.exec(textbox.value.substr(0, selectionStart)).toString();
			this.detected = textbox.value.substring(j, selectionStart);
			return this.detected;
		},
		replace: function (index) {
			if (QuickFoxNotes.wordList.length < index) {
				QuickFoxNotes.suggestMode = 0;
				return;
			}
			var textbox = QuickFoxNotes.getEditor();	//Selected Textbox
			var selectionStart = textbox.selectionStart;
			var selectionEnd = textbox.selectionEnd;
			
			var data = QuickFoxNotes.wordList[index];

			if (data.indexOf(this.detected) == 0) {	//The suggested word matched
				data = data.substr(this.detected.length);
				QuickFoxNotes.insertTextAtCursorPoint (data, textbox, selectionStart, selectionEnd);
				textbox.selectionStart = selectionStart;
				textbox.selectionEnd = selectionStart + data.length;
			}
			else {
				QuickFoxNotes.insertTextAtCursorPoint (data, textbox, selectionStart - this.detected.length, selectionEnd);
				textbox.selectionStart = selectionStart - this.detected.length;
				textbox.selectionEnd = selectionStart - this.detected.length + data.length;
			}
		}
	},
	/**Contextmenu*/
	contextmenu: {
		doPopupItemEnabling: function (){	//This function only check menu & menuitems; deep level 1
			function applyCommand(element, command) {
				var controller = document.commandDispatcher.getControllerForCommand(command);
				if (controller) {
					var enabled = controller.isCommandEnabled(command);
					if (enabled)
						element.removeAttribute("disabled");
					else
						element.setAttribute("disabled", "true");
				}
				else
					element.setAttribute("disabled", "true");
			}
			var popupNode= _("editor_menupopup");

			var children = popupNode.childNodes;
			for (var i = 0; i < children.length; i++) {
				var child = children[i];

				if (!child.getAttribute("cmd") && child.childNodes.length) {	//There are some menus which should be disabled itself not child elements
					
					for (var j = 0; j < child.childNodes[0].childNodes.length; j++) {
						var command = child.childNodes[0].childNodes[j].getAttribute("cmd");
						if (command)
							applyCommand(child.childNodes[0].childNodes[j], command);
					}
				}
				else {
					var command = child.getAttribute("cmd");
					if (command)
						applyCommand(child, command);
				}
			}
		},
		goDoCommand:function (command) {
			try {
				var controller = top.document.commandDispatcher.getControllerForCommand(command);
				if ( controller && controller.isCommandEnabled(command))
					controller.doCommand(command);
			}
			catch (e) {QuickFoxNotes.jsDump.logStringMessage("QuickFox Notes: [e][goDoCommand][e:" + e.message + "][command:" + command + "]");}
		},
		
		controller: {
			supportsCommand : function(cmd){
				switch(cmd) {
					case "cmd_qfn_addToDic":
					case "cmd_copy":	//Overwrite original copy command
					case "cmd_cut":		//Overwrite original cut command
					case "cmd_qfn_refreshClipboard": //Refresh Clipboard list
					case "cmd_qfn_copyHistory":
					case "cmd_qfn_email":
					case "cmd_qfn_print":
					case "cmd_qfn_addTime":
					case "cmd_qfn_encrypt":
					case "cmd_qfn_decrypt":
					case "cmd_qfn_google1":
					case "cmd_qfn_google2":
					case "cmd_qfn_google3":
					case "cmd_qfn_google4":
					case "cmd_qfn_google5":
					case "cmd_qfn_insertSymbol":
						return true;
					default:
						return false;
				}
			},
			isCommandEnabled : function(cmd) {
				var textbox = QuickFoxNotes.getEditor();
				var isTextReadonly = (textbox.getAttribute('readonly') == 'true');
				
				switch(cmd) {
					case "cmd_qfn_insertSymbol":
					case "cmd_qfn_addTime":
						return !isTextReadonly;
					case "cmd_qfn_addToDic":
						return true;
					case "cmd_copy":	//Overwrite original copy command
						return textbox.editor.canCopy();
					case "cmd_cut":		//Overwrite original cut command
						return textbox.editor.canCut();
					case "cmd_qfn_copyHistory":
						return !isTextReadonly && (QuickFoxNotes.copyHistory.length != 0);
					case "cmd_qfn_email":
					case "cmd_qfn_print":
						return (textbox.value != "");
					case "cmd_qfn_encrypt":
						return !isTextReadonly && (textbox.value != "" && !/^QFN_fGfsEawFesdfawM/.test(textbox.value));
					case "cmd_qfn_decrypt":
						return !isTextReadonly && (textbox.value != "" && /^QFN_fGfsEawFesdfawM/.test(textbox.value));
					case "cmd_qfn_google1":
					case "cmd_qfn_google2":
					case "cmd_qfn_google3":
					case "cmd_qfn_google4":
						QuickFoxNotes.suggestWord.detectWord (1);
						return (!/^[\ \t]*$/.test(QuickFoxNotes.suggestWord.detected));
					case "cmd_qfn_google5":
						return (textbox.selectionEnd != textbox.selectionStart);
					default:
						return true;
				}
			},
			doCommand : function(cmd) {
				switch(cmd) {
					case "cmd_qfn_addToDic":
						document.popupNode.parentNode.spellCheckerUI.addToDictionary();
						break;
					case "cmd_copy":	//Overwrite original copy command
					case "cmd_cut":		//Overwrite original cut command
					case "cmd_qfn_refreshClipboard":		//Modify Clipboard
						var textbox = QuickFoxNotes.getEditor();
						var selectedText = textbox.editor.selection.toString();	//Do it before cut!
						//Do the old job
						if (cmd == "cmd_copy")
							textbox.editor.copy();
						else if (cmd == "cmd_cut")
							textbox.editor.cut();
							
						//Internal Clipboard managing
						if (!/^[\ \t]*$/.test(selectedText) && cmd != "cmd_qfn_refreshClipboard") {	//Selected text is not space, tab
							//Remove the same old elements
							var i = QuickFoxNotes.copyHistory.indexOf(selectedText);
							while (i != -1) {
								var temp = QuickFoxNotes.copyHistory.splice(i);
								temp.shift();	//Remove first element
								QuickFoxNotes.copyHistory = QuickFoxNotes.copyHistory.concat(temp);
								i = QuickFoxNotes.copyHistory.indexOf(selectedText);
							}
						
							QuickFoxNotes.copyHistory.unshift(selectedText);	//Add selected text
						
							const len = QuickFoxNotes.prefs.getIntPref("copyHistory");
						
							while (QuickFoxNotes.copyHistory.length > len)
								QuickFoxNotes.copyHistory.pop(); //remove the last element
						}
						//Add items to menu
						var menupopup = _("editor_menupopup_copyHistory");
						
						function addNewChild (data) {
							var menuitem = document.createElement("menuitem");
							if (data.length > 20)
								menuitem.setAttribute("label", data.substr(0,20) + "...");
							else
								menuitem.setAttribute("label", data);
							menuitem.setAttribute("value", data);
							menuitem.setAttribute("tooltiptext", data);
							menuitem.setAttribute("oncommand", "QuickFoxNotes.insertTextAtCursorPoint(this.value)");
							
							menupopup.insertBefore(menuitem, _("editor_clipboard_clearall_separator"));
						}
						
						while (menupopup.childNodes.length - 2 > QuickFoxNotes.copyHistory.length)	//@ last elements are for 'clear all'
							menupopup.removeChild(menupopup.firstChild);
						for (var i = 0; i < QuickFoxNotes.copyHistory.length; i++) {
							if (menupopup.childNodes.length - 2 > i) {
								if (QuickFoxNotes.copyHistory[i].length > 20)
									menupopup.childNodes[i].setAttribute("label", QuickFoxNotes.copyHistory[i].substr(0, 20) + "...");
								else
									menupopup.childNodes[i].setAttribute("label", QuickFoxNotes.copyHistory[i]);
								menupopup.childNodes[i].setAttribute("value", QuickFoxNotes.copyHistory[i]);
								menupopup.childNodes[i].setAttribute("tooltiptext", QuickFoxNotes.copyHistory[i]);
							}
							else
								addNewChild(QuickFoxNotes.copyHistory[i]);
						}
						break;
					case "cmd_qfn_copyHistory":
						break;
					case "cmd_qfn_email":
						QuickFoxNotes.email.open();
						break;
					case "cmd_qfn_print":
						QuickFoxNotes.createReport();
						break;
					case "cmd_qfn_addTime":
						QuickFoxNotes.insertTextAtCursorPoint(QuickFoxNotes.getTime());
						break;
					case "cmd_qfn_encrypt":
						QuickFoxNotes.secure.encrypt();
						break;
					case "cmd_qfn_decrypt":
						QuickFoxNotes.secure.decrypt();
						break;
					case "cmd_qfn_google1":
						QuickFoxNotes.suggestWord.detectWord (1); 
						QuickFoxNotes.suggestWord.suggest();
						break;
					case "cmd_qfn_google2":
						QuickFoxNotes.suggestWord.detectWord (2); 
						QuickFoxNotes.suggestWord.suggest();
						break;
					case "cmd_qfn_google3":
						QuickFoxNotes.suggestWord.detectWord (3); 
						QuickFoxNotes.suggestWord.suggest();
						break;
					case "cmd_qfn_google4":
						QuickFoxNotes.suggestWord.detectWord (4); 
						QuickFoxNotes.suggestWord.suggest();
						break;
					case "cmd_qfn_google5":
						var textbox = QuickFoxNotes.getEditor();
						const selectionStart = textbox.selectionStart;
						textbox.selectionStart = textbox.selectionEnd;
						QuickFoxNotes.suggestWord.detected = textbox.value.substring(selectionStart, textbox.selectionEnd);
						QuickFoxNotes.suggestWord.suggest();
				}
			},
			onEvent : function(evt){}
		}
	},
	onpopupshowing: function (me, e) {
		if(e.target.id != "editor_menupopup")	//This is called from sub menu not main menu
			return true;
	
		var textbox = document.popupNode.parentNode;
		textbox.spellCheckerUI.initFromEvent(document.popupRangeParent, document.popupRangeOffset);
 
		//Remove old results 
		for (var i = 0; i < me.childNodes.length; i ++) {
			if (me.childNodes[i].id == "editor_menupopup_dictionary")
				break;
		}
		for (var j = 0; j < i; j++) {
			me.removeChild(me.firstChild);
		}

		var insertBefore = _('editor_menupopup_dictionary');
		var numsug = 0;
		if (this.prefs.getBoolPref("SpellCheck"))
			numsug = textbox.spellCheckerUI.addSuggestionsToMenu(me, insertBefore, 5);
		
		if (!numsug) {
			_("editor_menupopup_dictionary").style.visibility='collapse';
			_("editor_menupopup_dictionary_sep").style.visibility='collapse';
		}
		else {
			_("editor_menupopup_dictionary").style.visibility='visible';
			_("editor_menupopup_dictionary_sep").style.visibility='visible';
		}
		this.contextmenu.doPopupItemEnabling();
		
		return true;
	},
	/**Drag & Drop*/
	dragNdrop: {
		dragObserver: {
			onDragStart: function (event, transferData, action) {
				var txt = event.target.parentNode.selectedIndex;
				transferData.data = new TransferData();
				transferData.data.addDataForFlavour("text/unicode/QFN", txt);
			},
			onDragExit: function (event, transferData, action) {
				_('tabs-drop-indicator-bar').collapsed = true;
			}
		},
		
		dropObserver: {
			getSupportedFlavours : function () {
				var flavours = new FlavourSet();
				flavours.appendFlavour("text/unicode/QFN");
				return flavours;
			},
			onDragOver: function (event, flavour, session) {
				var target = event.target;
				var indicator = _("tabs-drop-indicator");
				_("tabs-drop-indicator-bar").collapsed = false;
				indicator.style.MozMarginStart = target.boxObject.x - indicator.boxObject.width / 2 + "px";
			},
		 
			onDrop: function (event, dropdata, session) {
				_('tabs-drop-indicator-bar').collapsed = true;
				
				var refIndex = parseInt(dropdata.data);
					
				var tabRef = event.target;
				var tabSource = QuickFoxNotes.tabs.getItemAtIndex(refIndex);

				if(tabRef.id == "views-tbar-spacer")
				 tabRef = null;
			   
				var tabpanelSource = QuickFoxNotes.tabpanels.childNodes[refIndex];
				
				var tabRefItem = QuickFoxNotes.tabs.getIndexOfItem(tabRef);
				if (tabRefItem == 0) {	//Dummy note is selected!
					tabRefItem += 1;
					tabRef = tabRef.nextSibling;
				}
				var tabpanelRef = null;
				if (tabRef)
					tabpanelRef = QuickFoxNotes.tabpanels.childNodes[tabRefItem];
					
				//Save information about source tab
				const value = tabpanelSource.getElementsByClassName("notepad")[0].value;
				const scrollTop = tabpanelSource.getElementsByClassName("notepad")[0].inputField.scrollTop;
				const selectionStart = tabpanelSource.getElementsByClassName("notepad")[0].selectionStart;
				const selectionEnd = tabpanelSource.getElementsByClassName("notepad")[0].selectionEnd;
				const saveMe = tabpanelSource.getElementsByClassName("notepad")[0].saveMe;
				const isReadonly = tabpanelSource.getElementsByClassName("notepad")[0].getAttribute('readonly') == 'true';

				QuickFoxNotes.tabpanels.insertBefore (tabpanelSource, tabpanelRef);
				QuickFoxNotes.tabs.insertBefore (tabSource, tabRef);
				//Restore source tab
				tabpanelSource.getElementsByClassName("notepad")[0].value = value;
				tabpanelSource.getElementsByClassName("notepad")[0].inputField.scrollTop = scrollTop;
				tabpanelSource.getElementsByClassName("notepad")[0].selectionStart = selectionStart;
				tabpanelSource.getElementsByClassName("notepad")[0].selectionEnd = selectionEnd;
				tabpanelSource.getElementsByClassName("notepad")[0].saveMe = saveMe;
				if (isReadonly) 
					tabpanelSource.getElementsByClassName("notepad")[0].setAttribute('readonly', true);
				tabSource.modifyPosition = true;
				tabpanelSource.getElementsByClassName("notepad")[0].controllers.insertControllerAt(0, QuickFoxNotes.contextmenu.controller);				
				//Refresh List
				var tempIndex = QuickFoxNotes.tabbox.selectedIndex;
				QuickFoxNotes.tabpanels.selectedIndex = 0;
				QuickFoxNotes.tabpanels.selectedIndex = tempIndex;
				QuickFoxNotes.renderTabs();
			}
		}
	}
}