/*
* QuickFox Notes by Amin E (InBasic)
* eMail: inb.cor@gmail.com
*
* Source Code License: Mozilla Public License, version 1.1
*/

QuickFoxNotes.initialize = function () {
	try {
		//Initilizing elements
		this.window = window;
		this.tabs = _("editor-tabbox-tabs");
		this.tabpanels = _("editor-tabbox-tabpanels");
		this.tabbox = _("editor-tabbox");
		this.notifyBox = _('notificationbox');
				
		this.stringsBundle = _("editor-bundle");
		//Initialize simplenote interface
		this.simplenote.initialize();
		//Initialize io
		this.io.initialize();
		//add panels from bookmarks
		this.constructPanels();	 
		//Restore Clipboard history
		try {
			var clipboard = this.prefs.getComplexValue("clipboard", Components.interfaces.nsISupportsString).data;
			if (clipboard)
				this.copyHistory = this.nativeJSON.decode(clipboard);
		}
		catch(e) {}
		QuickFoxNotes.contextmenu.controller.doCommand("cmd_qfn_refreshClipboard");
		//Restore Trash
		try {
			var menupopup = _("editor.toolbar7.menupopup");
			var trashData = this.nativeJSON.decode(
								this.prefs.getComplexValue("trashData", Components.interfaces.nsISupportsString).data);
			
			for (var i = 0; i < trashData.length; i++) {	// trashData.length is 1 or more!
				var menuitem = document.createElement("menuitem");
				menuitem.setAttribute("label", trashData [i].label);
				menuitem.setAttribute("value", trashData [i].value);
				menuitem.setAttribute("oncommand", this.trashCommand);
				menupopup.appendChild(menuitem);
				menupopup.parentNode.disabled = false; //Make sure that menupopup is enabled
			}
		} catch(e) {}
		//Add empty Trash element
		var menuseparator = document.createElement("menuseparator");
		menuseparator.setAttribute("id",  "editor.toolbar7.menupopup.trash");
		menupopup.appendChild(menuseparator);
		var menuitem = document.createElement("menuitem");
		menuitem.setAttribute("label",  this.stringsBundle.getString('emptyTrash'));
		menuitem.setAttribute("oncommand", "QuickFoxNotes.clearTrash();");
		menupopup.appendChild(menuitem);
		//Initialize toolbar
		this.toolbar.initialize();
		//Set Always on top
		this.alwaysOnTop(window, this.prefs.getBoolPref("alwaysOnTop"));
		
		if (this.prefs.getBoolPref("Autosave"))
			this.timer(true); //Timer to save all of the notes after definded interval
		
		//Select font
		var fontSize = parseInt(this.prefs.getCharPref("FontSize"));
		var fontIndex = (fontSize - 8) / 2;  //8 is first size!
		try {
			_('editor-tabbox-toolbar-menu1-radiogroup').childNodes[fontIndex].setAttribute("checked", true);
		} catch(e) {QuickFoxNotes.jsDump.logStringMessage("QuickFox Notes: [e][fontIndex][e: out of range fontIndex]");}
		//Select tab width
		var tabWidth = parseInt(this.prefs.getCharPref("TabWidth"));
		var widthIndex = (tabWidth - 50) / 10;  //50 is first size!
		try {
			_('editor-tabbox-toolbar-menu3-radiogroup').childNodes[widthIndex].setAttribute("checked", true);
		} catch(e) {QuickFoxNotes.jsDump.logStringMessage("QuickFox Notes: [e][widthIndex][e: out of range widthIndex]");}
		//Select tab color
		var TabColor = this.prefs.getCharPref("TabColor");
		var menu4 = _('editor-tabbox-toolbar-menu4-radiogroup');
		for (i = 0; i < menu4.childNodes.length; i++) {
			if (menu4.childNodes[i].getAttribute("label") == TabColor)
				menu4.childNodes[i].setAttribute("checked", true);
		}
		//Select tab text color
		var TextColor = this.prefs.getCharPref("TextColor");
		var menu6 = _('editor-tabbox-toolbar-menu6-radiogroup');
		for (i = 0; i < menu6.childNodes.length; i++) {
			if (menu6.childNodes[i].getAttribute("label") == TextColor)
				menu6.childNodes[i].setAttribute("checked", true);
		}
		//Select reminder
		var sec = this.prefs.getIntPref("ReminderSec");
		var secIndex = (sec / 10) + ((sec / 10) ? 1 : 0); //there is a seperator between item 0 and 1
		try {
			_('editor-tabbox-toolbar-menu5-radiogroup').childNodes[secIndex].setAttribute("checked", true);
		} catch(e) {QuickFoxNotes.jsDump.logStringMessage("QuickFox Notes: [e][secIndex][e: out of range secIndex]");}
		//Select reminder window type
		var reminderIndex = this.prefs.getIntPref("reminderWindowType");
		_('editor-tabbox-toolbar-menu5-menu-radiogroup').childNodes[reminderIndex].setAttribute("checked", true);
		//Select time format
		var timeFormat = this.prefs.getBoolPref("timeFormat");
		_('editor-tabbox-toolbar-menu10-radiogroup').childNodes[timeFormat ? 0 : 1].setAttribute("checked", true);
		//Select group for tabs
		const group2 = this.prefs.getBoolPref("showGroupInTab");
		_('editor-tabbox-toolbar-menu11-radiogroup2').childNodes[group2 ? 0 : 1].setAttribute("checked", true);
		//Select group for archive
		const group1 = this.prefs.getBoolPref("showGroupInArchive");
		_('editor-tabbox-toolbar-menu11-radiogroup1').childNodes[group1 ? 0 : 1].setAttribute("checked", true);
		//Select scroll type
		var scrollIndex = (this.prefs.getBoolPref("ScrollStopOnEnd") ? 0 : 1);
		_('editor-tabbox-toolbar-menu2-radiogroup').childNodes[scrollIndex].setAttribute("checked", true);
		this.scrollOnMouse();
		//Disabling Set on Top when QFN is minimized
		//window.addEventListener("resize", QuickFoxNotes.resize, false);
		//Do commands
		var argument = this.prefs.getComplexValue("argument", Components.interfaces.nsISupportsString).data;
		if (argument) 
			this.prefs.setCharPref("argument", "");

		if(argument || ('arguments' in window && window.arguments.length > 0)) {
			var command = argument ? argument : window.arguments[0];
			if (command && command.match(/^[^ ]*/) == "openFile") {
				var filePath = command.match(/[^ ]*$/).toString();
				
				var file = Components.classes["@mozilla.org/file/local;1"]
					.createInstance(Components.interfaces.nsILocalFile);
				
				file.initWithPath(filePath);
				var obj = this.addSingleTab();
				obj._textbox.focus();	//This is set focus to newly created tab so importNote print in write note
				var tmp = /(.*)\.[^\.]*$/.exec(file.leafName)
				obj._tab.label = (tmp ? tmp[1] : file.leafName);
				this.importText(file);
			}
		}
		//Install agent-Sheet
		if (this.prefs.getCharPref("agentSheet")) {
			var sss = Components.classes["@mozilla.org/content/style-sheet-service;1"]
								.getService(Components.interfaces.nsIStyleSheetService);
			var ios = Components.classes["@mozilla.org/network/io-service;1"]
								.getService(Components.interfaces.nsIIOService);
			var uri = ios.newURI("data:text/css, " + this.prefs.getCharPref("agentSheet"), null, null);
			if(!sss.sheetRegistered(uri, sss.AGENT_SHEET))
			  sss.loadAndRegisterSheet(uri, sss.AGENT_SHEET);
		}
		//Do backup
		setTimeout(function(){QuickFoxNotes.backups.init();}, 500);
		//Api
		setTimeout(function(){QuickFoxNotes.api.initialize();}, this.prefs.getIntPref("scriptsInitTime"));
		//Listener for notification box; Back to the normal mode whenever user close notification by mouse!
		this.notifyBox.addEventListener('command', function(){
			if(!QuickFoxNotes.notifyBox.allNotifications.length) {
				QuickFoxNotes.suggestMode = 0;
			}
		}, false);			
 	}
	catch (e) {
		this.errorMode = true;
		alert('IMPORTANT NOTE: \"QuickFox Notes\" experienced an error during initilization; Because of the potential danger, QFN will \"NOT\" save any change within current session.' +
			  '\n\nError report: \n' +
			  '{' + e.name + " @ " + e.message + '\n' + e.lineNumber + ' : ' + e.fileName + '}\n\n' +
			  'Please send the error report!');
	}
};