/*
* QuickFox Notes by Amin E (InBasic)
* eMail: inb.cor@gmail.com
*
* Source Code License: Mozilla Public License, version 1.1
*/

/**Set attributes of an element*/
QuickFoxNotes.setAttributes = function(element, list) {	//list is [['att1', 'value1'], [,]]
	for (var i = 0; i < list.length; i++) 
		element.setAttribute(list[i][0], list[i][1]);
};
/**Do key*/
QuickFoxNotes.doKey = function(aKey) {
	_(aKey).doCommand();
};
QuickFoxNotes.getEditor = function (index) {
	if (!index)
		return this.tabpanels.selectedPanel.getElementsByClassName('notepad')[0];
	else
		return this.tabpanels.childNodes[index].getElementsByClassName("notepad")[0];
};
QuickFoxNotes.getFontFamily = function (index) {
	try {

		return window.getComputedStyle(this.getEditor(index), null).getPropertyValue('font-family').match(/[^\"]+/).toString();	//In linux name comes with qoutations
	}
	catch (e) {return "";}
};
/**Changing reminder*/
QuickFoxNotes.changeReminder = function (sec) {
	this.prefs.setIntPref("ReminderSec", sec);
};
/**Statusbar icon*/
QuickFoxNotes.statusbarIcon = function (type, show) {
	var changeIconStatus = function (id, show) {
		var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
			.getService(Components.interfaces.nsIWindowMediator);
		var enumerator = wm.getEnumerator("navigator:browser");
		while(enumerator.hasMoreElements()) {
			var win = enumerator.getNext();
			win.document.getElementById(id).hidden = !show;
		}
	}

	switch (type) {
	case "window":
		this.prefs.setBoolPref('statusbarWindowIcon', show);
		changeIconStatus ("quickfox-notes-statusbarpanel1", show);
		break;
	case "tab":
		this.prefs.setBoolPref('statusbarTabIcon', show);
		changeIconStatus ("quickfox-notes-statusbarpanel2", show);
		break;
	case "appcontent":
		this.prefs.setBoolPref('statusbarAppcontentIcon', show);
		changeIconStatus ("quickfox-notes-statusbarpanel3", show);
		break;
	}
	
	return this;
};
/**Put editor to top most*/
QuickFoxNotes.alwaysOnTop = function (win, bol) {	//type can be raisedZ, normalZ, loweredZ
	try {	//when QFN opend in tab it can not put itself to the top most layer
		var xulWin = win.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
		   .getInterface(Components.interfaces.nsIWebNavigation).QueryInterface(Components.interfaces.nsIDocShellTreeItem)
		   .treeOwner.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
		   .getInterface(Components.interfaces.nsIXULWindow);
		xulWin.zLevel = xulWin[ bol ? "raisedZ" : "normalZ"];
	} catch (e) {this.jsDump.logStringMessage("QuickFox Notes: [e][alwaysOnTop][e:" + e.message + "]");}
	this.prefs.setBoolPref("alwaysOnTop", bol);
	
	return this;
};
/**Changing font size of notes*/
QuickFoxNotes.changeFontSize = function (pixel) {
	this.prefs.setCharPref("FontSize", pixel);
	for (var i = 0; i < this.currentNumberOfPanels; i++) {
		this.getEditor(i).style.fontSize = pixel + "px";
	}
};
/**Changing tab's width*/
QuickFoxNotes.changeTabWidth = function (pixel) {
	this.prefs.setCharPref("TabWidth", pixel);
	for (var i = 0; i < this.currentNumberOfPanels; i++) {
		this.tabs.getElementsByTagName("tab")[i].style.minWidth = pixel + "px";
	}
};
/**Changing tab's color*/
QuickFoxNotes.changeTabColor = function (color) {
	this.prefs.setCharPref("TabColor", color);
	for (var i = 0; i < this.currentNumberOfPanels; i++) {
		this.getEditor(i).style.backgroundColor = color;
		this.tabpanels.getElementsByTagName("tabpanel")[i].style.backgroundColor = color;
	}
};
/**Changing tab's text color*/
QuickFoxNotes.changeTextColor = function (color) {
	this.prefs.setCharPref("TextColor", color);
	for (var i = 0; i < this.currentNumberOfPanels; i++) {
		this.getEditor(i).style.color = color;
	}
};
/**Activate or disactivate Word wraping*/
QuickFoxNotes.wordWrap = function (bol) {
	this.prefs.setBoolPref("wordWrap", bol);
	for (var i = 0; i < this.currentNumberOfPanels; i++) {
		//setAttribute("spellcheck", bol) refreshed the contextmenu, so new items will disappear
		this.getEditor(i).setAttribute("wrap", bol ? "on" : "off");
	}
};
/**Activate or disactivate spellchecking*/
QuickFoxNotes.spellCheck = function (bol) {
	this.prefs.setBoolPref("SpellCheck", bol);
	for (var i = 0; i < this.currentNumberOfPanels; i++) {
		//setAttribute("spellcheck", bol) refreshed the contextmenu, so new items will disappear
		this.getEditor(i).editor.setSpellcheckUserOverride(bol);
	}
};
/**Hide popup when Enter is pressed*/
QuickFoxNotes.handleReturnKey = function (event) {
	if (event.keyCode == event.DOM_VK_RETURN) 
		_("editor-context").hidePopup();
};
/**Timer to save all data in definite interval*/
QuickFoxNotes.timer = function (bol) {
	if (this.intervalID && bol)
		return;	//Prevent double call

	this.prefs.setBoolPref("Autosave", bol);
	
	if (bol)
		this.intervalID = window.setInterval(function () {QuickFoxNotes.savePanels()}, this.prefs.getIntPref("TimeInterval"));
	else {
		window.clearInterval(this.intervalID);
		this.intervalID = null;
	}
};
/**Notify a massage*/
QuickFoxNotes.notify = function (msg, image, hideAfter) {
	this.notifyBox.removeAllNotifications( true );
	var img = (image == "google") ? "chrome://qfnotes/skin/google.png" : null;
	this.notifyBox.appendNotification( msg , null , img , this.notifyBox.PRIORITY_INFO_MEDIUM , null );

	if (hideAfter)
		setTimeout(function(){ QuickFoxNotes.notifyBox.removeAllNotifications( true );}, hideAfter);
};
/**Open links*/
QuickFoxNotes.openLink = function (url) {
	var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
		   .getService(Components.interfaces.nsIWindowMediator);
	var win = wm.getMostRecentWindow("navigator:browser");
	var gBrowser = win.getBrowser();
	gBrowser.loadOneTab(url, null, null, null, false, false);
	win.focus();
};
/**Get current time*/
QuickFoxNotes.getTime = function () {
	var currentTime = new Date();

	var hours = currentTime.getHours();
	
	var temp = "";
	if (this.prefs.getBoolPref("timeFormat")) {
		var temp = "AM";
		if (hours > 12) {
			hours -= 12;
			temp = "PM"
		}
	}
	hours = ((hours < 10) ? "0" : "") + hours;
	var minutes = currentTime.getMinutes();
	minutes = ((minutes < 10) ? "0" : "") + minutes;
	var month = currentTime.getMonth() + 1
	month = ((month < 10) ? "0" : "") + month;
	var day = currentTime.getDate()
	day = ((day < 10) ? "0" : "") + day;
	var year = currentTime.getFullYear();

	return hours + ":" + minutes + " " + temp + (temp ? " " : "") + day + "/" + month + "/" + year;
};
/**email service*/
QuickFoxNotes.email = {
	open: function () {
		var data = QuickFoxNotes.getEditor().value;
		
		openDialog("chrome://qfnotes/content/emailSettings.xul", "dlg", "", 
			QuickFoxNotes.group.read(QuickFoxNotes.tabs.selectedItem),
			data, QuickFoxNotes.getFontFamily(), QuickFoxNotes.prefs.getCharPref("FontSize"));
	}
};
/**add-on manager*/
QuickFoxNotes.addons = {
	open: function () {
		openDialog("chrome://qfnotes/content/addonManager.xul", "dlg,centerscreen,resizable=yes", "modal");
	}
};
/**Create Report*/
QuickFoxNotes.createReport = function () {
	//Creating temporary file
	var file = Components.classes["@mozilla.org/file/directory_service;1"].
						 getService(Components.interfaces.nsIProperties).
						 get("TmpD", Components.interfaces.nsIFile);
	file.append("qfn_report.html");
	file.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0666);
	//Write data to a file
	var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].
		 createInstance(Components.interfaces.nsIFileOutputStream);

	foStream.init(file, 0x02 | 0x08 | 0x20, 0666, 0); 
	var converter = Components.classes["@mozilla.org/intl/converter-output-stream;1"].
					  createInstance(Components.interfaces.nsIConverterOutputStream);
	converter.init(foStream, "UTF-8", 0, 0);
	var data = QuickFoxNotes.getEditor().value;
	//Convert text to valid html file
	data = "<html><head><meta http-equiv=\"Content-Type\" content=\"text/html;charset=UTF-8\"><title>" + 
		   this.group.read(this.tabs.selectedItem) + 
		   "</title></head><body onLoad='window.print();'><pre STYLE=\"white-space: -moz-pre-wrap; white-space: pre-wrap; font-family: '" + QuickFoxNotes.getFontFamily() + "'; font-size: " + QuickFoxNotes.prefs.getCharPref("FontSize") +  "px;\">" +
		   data.replace(/&/g, "&amp;").replace(/>/g, "&gt;").replace(/</g, "&lt;").replace(/"/g, "&quot;") + 
		   "</pre></body></html>";
	
	converter.writeString(data);
	converter.close(); 
	//Convert path to URL
	var ios = Components.classes["@mozilla.org/network/io-service;1"].  
						getService(Components.interfaces.nsIIOService);  
	var URL = ios.newFileURI(file);
	
	var wn = window.open(URL.spec);
};
/**Handling double click on the text box*/
QuickFoxNotes.dbclick = function (textbox) {
	const value  = textbox.value;
	//Detect a word at cursor position
	var word = /[^\t\ \n]*$/.exec(value.substring(0, textbox.selectionStart))[0] +
		/[^\t\ \n]*/.exec(value.substring(textbox.selectionStart))[0];
	//Validate the link
	var data = word.match(/(ftp|http|https):\/\/(\w+:{0,1}\w*@)?([^ \'\"\f\n\r\t\v\u00A0\u2028\u2029]+)(:[0-9]+)?(\/|\/([\w#!:.?+=&%@!\-\/]))?/);
	if (!data)
		return;

	//Find the most recent window
	var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]  
		.getService(Components.interfaces.nsIWindowMediator);  
	var win = wm.getMostRecentWindow("navigator:browser");
	if (win) {
		if (this.prefs.getBoolPref("openLinkInBackground"))
			win.gBrowser.addTab(data[0]);
		else
			win.gBrowser.selectedTab = win.gBrowser.addTab(data[0]);
	}
	else 
		window.open(data[0]);
};
/**Change the font family*/
QuickFoxNotes.changeFontFamily = function () {
	var check = {value: false};
	var input = {value: this.prefs.getComplexValue("FontFamily", Components.interfaces.nsISupportsString).data};
	var result = this.prompts.prompt(null, this.stringsBundle.getString('changeFontFamilyTitle'), this.stringsBundle.getString('changeFontFamily'), input, null, check);

	if (result) {
		var str = Components.classes["@mozilla.org/supports-string;1"]
			.createInstance(Components.interfaces.nsISupportsString);
			str.data = input.value;
		this.prefs.setComplexValue("FontFamily", Components.interfaces.nsISupportsString, str);
		alert(this.stringsBundle.getString('fontFamilySuccess'));
		//Closing QFN
		this.closeMe();
	}
};