/*
* QuickFox Notes by Amin E (InBasic)
* eMail: inb.cor@gmail.com
*
* Source Code License: Mozilla Public License, version 1.1
*/

/**API Loader*/
QuickFoxNotes.api = {
	loadedGUIDs: [],	//This will check whether GUID is loaded or not
	pointers: [],
	
	initialize: function () {
		var arrayPath = this.scriptsPath();
		for (var i = 0; i < arrayPath.length; i++) {
			try {
				this.loadScript(arrayPath[i]);
			}
			catch(e) {
				alert("QFN::Script Loader Error:\n\nPath: " + arrayPath[i] + "\n" + e.name + " @ " + e.message + '\n' + e.lineNumber + ': ' + e.fileName);
			}
		}
	},
	amIdisabled: function (GUID) {
		var blackList = [];
		try {
			blackList = QuickFoxNotes.nativeJSON.decode(QuickFoxNotes.prefs.getCharPref("addonsBlacklist"));
		} catch (e) {}
		return (blackList.indexOf(GUID) != -1);
	},
	scriptsPath: function () {	//Periority: User folder Z>A, Chrome folder
		var arrayPath = [];
	
		var path = QuickFoxNotes.prefs.getComplexValue("scriptsPath", Components.interfaces.nsISupportsString).data;
		if (path) {
			var file = Components.classes["@mozilla.org/file/local;1"].
					   createInstance(Components.interfaces.nsILocalFile);
			file.initWithPath(path);
			//Enumerate files:
			var entries = file.directoryEntries;
			while(entries.hasMoreElements()) {
				var entry = entries.getNext();
				entry.QueryInterface(Components.interfaces.nsIFile);

				var ios = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
				var URL = ios.newFileURI(entry);
				if (/\.js$/.test(URL.spec))
					arrayPath.push(URL.spec);
			}
			arrayPath.sort(function(a, b){return b>a})	//Sort array from Z>A so file with bigger version overwrite file with smaller version
		}
		else {
			//Create scripts folder
			var file = Components.classes["@mozilla.org/file/directory_service;1"].
					   getService(Components.interfaces.nsIProperties).
					   get("ProfD", Components.interfaces.nsIFile);
			file.append("qfn-scripts");
			if( !file.exists() || !file.isDirectory() ) {   // if it doesn't exist, create
			   file.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0777);
			}
			//Set scriptPath
			var str = Components.classes["@mozilla.org/supports-string;1"]
				.createInstance(Components.interfaces.nsISupportsString);
			str.data = file.path;
			QuickFoxNotes.prefs.setComplexValue("scriptsPath", Components.interfaces.nsISupportsString, str); 	
		}
		
		return arrayPath;
	},
	/**Initialize script with path*/
	loadScript: function(path) {	
		//Load Script
		var scope = {};
		Components.utils.import(path, scope);
		var GUID = scope.extLoader.info.GUID;
		if (this.loadedGUIDs.indexOf(GUID) != -1)	//This GUID is already loaded!
			return;
		this.loadedGUIDs.push(GUID);
		
		this.pointers[this.pointers.length] = scope;
		//Check if addon is disabled or not
		if (this.amIdisabled(GUID))
			return;
		//Addon is enabled
		scope.extLoader.pointers = {window: window, document: document, QFN: QuickFoxNotes, QuickFoxNotes: QuickFoxNotes, api: QuickFoxNotes.api};
		//Install UI
		if ("xul" in scope.extLoader) {
			const before = "data:text/xul, " +
						   (("css" in scope.extLoader) ? "<?xml-stylesheet href='data:text/css, " + scope.extLoader.css + "' type='text/css'?>" : "") +
							"<overlay id='" + GUID + "' xmlns:html='http://www.w3.org/1999/xhtml' \
							xmlns:rdf='http://www.w3.org/1999/02/22-rdf-syntax-ns#' \
							xmlns='http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul'>";
			const after = "</overlay>";
			try {
				this.overlayLoader.add(
					before + this.introMe(scope.extLoader.xul) + after,
					function() {scope.extLoader.initialize();}	//Load initializer after loading of the content
				);
			}
			catch(e) {
				alert("QFN::XUL Loader Error:\n\nPath: " + path + "\n" + e.name + " @ " + e.message + '\n' + e.lineNumber + ': ' + e.fileName);
			}
		}
		else
			scope.extLoader.initialize();	//There is no content so load initializer independently
	},
	/**https://bugzilla.mozilla.org/show_bug.cgi?id=330458*/
	overlayLoader: {
		queue: [],
		loading: false,

		add: function (path, obs) {
			this.queue.push([path, obs]);
			this.load();
		},
		load: function() {
			if(!QuickFoxNotes.api.overlayLoader.queue.length ) return;

			if (!QuickFoxNotes.api.overlayLoader.loading) {
				QuickFoxNotes.api.overlayLoader.loading=true;
				document.loadOverlay(QuickFoxNotes.api.overlayLoader.queue[0][0], {
					observe:function(){
						QuickFoxNotes.api.overlayLoader.loading = false;
						QuickFoxNotes.api.overlayLoader.queue[0][1]();
						QuickFoxNotes.api.overlayLoader.queue.shift();
					}
				});
			}
			else {
				setTimeout(function(){QuickFoxNotes.api.overlayLoader.load()}, 100);
			}
		}
	},
	/**Introduce "me" pointer in scrpit*/
	introMe: function (str) {
		return str.replace(/\bme\b/g, "QuickFoxNotes.api.pointers[" + (this.pointers.length - 1) + "].extLoader");
	},
	/**This section contains all functions that are officially available to scripts*/
	list: {
		tab: function() {
			var tmpList = [];
			for (var i = 1; i < QuickFoxNotes.currentNumberOfPanels; i++)	//First Tab is dummy
				tmpList.push(QuickFoxNotes.tabs.getItemAtIndex(i));
			return tmpList;
		},
		note: function() {
			var tmpList = [];
			for (var i = 1; i < QuickFoxNotes.currentNumberOfPanels; i++)	//First Tab is dummy
				tmpList.push(QuickFoxNotes.getEditor(i));
			return tmpList;
		},
		archive: function() {
			var tmpList = [];
			for (var menuitem = document.getElementById("editor.toolbar6.menupopup").firstChild; menuitem; menuitem = menuitem.nextSibling)
				tmpList.push(menuitem);
			return tmpList;
		}
	},
	statusNotification: {
		id: null,

		show: function(txt) {
			if(this.id)
			clearTimeout(this.id)
			_('statusbar-display').value=txt;
			this.id = setTimeout(function(){_('statusbar-display').value='';}, QuickFoxNotes.prefs.getIntPref("statusNotification"))
		}
	},
	titleNotification:{
		id: null,

		show: function(txt) {
			if(this.id)
			clearTimeout(this.id)
			document.title = txt;
			this.id = setTimeout(function(){QuickFoxNotes.renderTabs()}, QuickFoxNotes.prefs.getIntPref("titleNotification"))
		}
	},
	alert: function(from, msg) {
		alert(from + ":\n\n" + msg);
	},
	//Get title and content of notes
	get: {
		noteTitle: function (tab) {
			if (!tab)
				tab = QuickFoxNotes.tabs.selectedItem;
			return QuickFoxNotes.group.readLabel(tab);
		},
		noteContent: function(index) {
			return QuickFoxNotes.getEditor(index).value;
		},
		archiveTitle: function(menuitem) {
			return QuickFoxNotes.group.readLabel(menuitem);
		},
		archiveContent: function(menuitem) {
			return menuitem.getAttribute("value");
		}
	},
	//Important locations path
	path: function (location) {		//Desk, Progs, ProfD, https://developer.mozilla.org/en/Code_snippets/File_I%2F%2FO
		var file = Components.classes["@mozilla.org/file/directory_service;1"].
			getService(Components.interfaces.nsIProperties).
			get(location, Components.interfaces.nsIFile);
			
		var osString = Components.classes["@mozilla.org/xre/app-info;1"]
			.getService(Components.interfaces.nsIXULRuntime).OS;
		
		return file.path + (/WIN/.test(osString) ? "\\" : "/");
	},
	//Save to file
	saveToFile: function(path, data) {		//Return true if operation is successful 
		//Create file
		var file = Components.classes["@mozilla.org/file/local;1"].
				   createInstance(Components.interfaces.nsILocalFile);
		try {
			file.initWithPath(path);
		}
		catch(e) {
			return false;
		}
		//Save file
		var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].
					   createInstance(Components.interfaces.nsIFileOutputStream);
		try {foStream.init(file, 0x02 | 0x08 | 0x20, 0666, 0);}
		catch(e) {return false;}
		var converter = Components.classes["@mozilla.org/intl/converter-output-stream;1"].
						createInstance(Components.interfaces.nsIConverterOutputStream);
		converter.init(foStream, "UTF-8", 0, 0);
		converter.writeString(data.replace(/\n/g, String.fromCharCode(13, 10)));
		converter.close(); // this closes foStream
		
		return true;
	},
	//Send a key combination to the editor
	sendKey: function (type, keycode, ctrl, alt, shift) {	//type: keypress, keydown, keyup
		var event = document.createEvent("KeyboardEvent");
		event.initKeyEvent(type, true, true, window, ctrl, alt, shift, false, keycode, keycode);

		QuickFoxNotes.getEditor().inputField.dispatchEvent(event); 
	},
	//Is QFN opened in Window mode
	isWindowMode: function (){
		var mainWindow =
			window.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
						   .getInterface(Components.interfaces.nsIWebNavigation)
						   .QueryInterface(Components.interfaces.nsIDocShellTreeItem)
						   .rootTreeItem
						   .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
						   .getInterface(Components.interfaces.nsIDOMWindow); 
				
		return (mainWindow == window);
	},
	//Get clipboard text
	getClipboard: function () {
		var clip = Components.classes["@mozilla.org/widget/clipboard;1"].getService(Components.interfaces.nsIClipboard);  
		if (!clip) return "";    
		var trans = Components.classes["@mozilla.org/widget/transferable;1"].createInstance(Components.interfaces.nsITransferable);  
		if (!trans) return "";  
		trans.addDataFlavor("text/unicode");  
		clip.getData(trans, clip.kGlobalClipboard);  
		   
		var str       = new Object();  
		var strLength = new Object();  
		  
		trans.getTransferData("text/unicode", str, strLength);  
		if (str) str = str.value.QueryInterface(Components.interfaces.nsISupportsString);  
		return str.data.substring(0, strLength.value / 2);
	},
	//Insert a text at cursor position
	insertTextAtCursorPoint: function (txt) {
		QuickFoxNotes.insertTextAtCursorPoint(txt);
	}
}